/***************************************************************************
                          qdatastoring.cpp  -  description
                             -------------------
    begin                : dim mar 17 16:56:00 CET 2003
    copyright            : (C) 2002 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

#include <qdom.h>
#include <qpopupmenu.h>

#include "qdatastoring.h"
#include "qpixmapboard.h"
#include "qthemeengine.h"
#include "qapplicationwindow.h"

QDataStoring::QDataStoring() 
{
  QString fn = homeDir + DIR_DELIMITER + "prefs.xml";
  QDomDocument doc( "Preferences" );
  QFile f( fn );
  if ( !f.open( IO_ReadOnly ) ) {
    // Store default value.
    checkDefaultPrefs();
    return;
  }
  if ( !doc.setContent( &f ) ) {
    // Store default value.
    f.close();
    checkDefaultPrefs();
    return;
  }
  f.close();  
  QDomElement current = doc.documentElement().firstChild().toElement();
  while (!current.isNull()) {
    QString att = current.attribute("attribute");
    QString val = current.attribute("value");
    prefs[att] = val;

    current = current.nextSibling().toElement();
  }
}

QDataStoring::~QDataStoring() {}

void QDataStoring::setBoard(QPixmapBoard *b) { qb = b; }
void QDataStoring::setApp(ApplicationWindow *aw) { main = aw; }

void QDataStoring::checkDefaultPrefs()
{
  prefs["window_size_width"] = "800";
  prefs["window_size_height"] = "600";
  prefs["window_maximized"] = "false";
  prefs["tiles_size"] = "small";
  prefs["with_random_ground"] = "true";
  prefs["theme"] = "Deathwing";
}

void QDataStoring::checkCurrentPrefs()
{
  prefs["window_size_width"] = QString::number(main->width());
  prefs["window_size_height"] = QString::number(main->height());
  prefs["window_maximized"] = (main->isMaximized())?"true":"false";
  prefs["tiles_size"] = (qb->bigpix)?"big":"small";
  prefs["with_random_ground"] = 
    (main->view->isItemChecked(main->buttonRandomGround))?"true":"false";
  prefs["theme"] = qb->theme->current.c_str();
}

void QDataStoring::SaveDeadTiles(QDomDocument &doc, QDomElement &liste)
{
  QDomElement deads = doc.createElement("dead_tiles");
  liste.appendChild(deads);
  QThemeEngine *th = qb->theme;
  for (map<int,int>::iterator it_theme=th->cases.begin();
       it_theme!=th->cases.end(); it_theme++) {
    if (it_theme->second==-2) {
      QDomElement dead = doc.createElement("dead_marine");
      dead.setAttribute("position",it_theme->first);
      deads.appendChild(dead);
    }
    else if (it_theme->second==-3) {
      QDomElement dead = doc.createElement("dead_genestealer");
      dead.setAttribute("position",it_theme->first);
      deads.appendChild(dead);
    }
  }
}
