/****************************************************************************
** $Id: smtp.cpp,v 1.6 2003/06/23 14:12:31 rom_vinot Exp $
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#include "smtp.h"

#include <qtextstream.h>
#include <qsocket.h>
#include <qdns.h>
#include <qtimer.h>
#include <qapplication.h>
#include <qmessagebox.h>

Smtp::Smtp( const QString &smtp, const QString &p, const QString &f, 
	    const QString &r, QString &msg )
  : message(msg), from(f), rcpt(r), port(p)
{
    socket = new QSocket( this );
    connect ( socket, SIGNAL( readyRead() ),
	      this, SLOT( readyRead() ) );
    //    connect (socket, SIGNAL( error(int) ),
    //	     this, SLOT( checkError(int) ) );

    dnsLookup = new QDns( smtp, QDns::A );
    connect( dnsLookup, SIGNAL(resultsReady()),
	     this, SLOT(dnsLookupHelper()) );

    state = Init;
    QTimer::singleShot(20000, this, SLOT(problemTimeOut()));
}


Smtp::~Smtp()
{
    delete t;
    delete socket;
}

void Smtp::doNothing() {}

//void Smtp::checkError(int error)
//{
//}

void Smtp::readyRead()
{
    // SMTP is line-oriented
    if ( !socket->canReadLine() )
	return;

    QString responseLine;
    do {
	responseLine = socket->readLine();
	response += responseLine;
    } while( socket->canReadLine() && responseLine[3] != ' ' );
    responseLine.truncate( 3 );

    if ( state == Init && responseLine[0] == '2' ) {
	// banner was okay, let's go on
	*t << "HELO there\r\n";
	state = Mail;
    } else if ( state == Mail && responseLine[0] == '2' ) {
	// HELO response was okay (well, it has to be)
	*t << "MAIL FROM: <" << from << ">\r\n";
	state = Rcpt;
    } else if ( state == Rcpt && responseLine[0] == '2' ) {
	*t << "RCPT TO: <" << rcpt << ">\r\n";
	state = Data;
    } else if ( state == Data && responseLine[0] == '2' ) {
	*t << "DATA\r\n";
	state = Body;
    } else if ( state == Body && responseLine[0] == '3' ) {
	*t << message << ".\r\n";
	state = Quit;
    } else if ( state == Quit && responseLine[0] == '2' ) {
	*t << "QUIT\r\n";
	// here, we just close.
	state = Close;
	//	emit status( tr( "Message sent" ) );
    } else if ( state == Close ) {
	// we ignore it
    } else {
	// something broke.
	QMessageBox::warning( qApp->activeWindow(),
			      tr( "Qt Mail Example" ),
			      tr( "Unexpected reply from SMTP server:\n\n" ) +
			      response );
	state = Close;
    }

    response = "";

    if ( state == Close ) {
      QMessageBox::about( qApp->activeWindow(), "QSpaceHulk, Mail send", 
			  "The mail has been send successfully.\n"
			  "You don't need to send the file manually.\n"
			  "(unless you've got an error message)" );
      QTimer::singleShot( 0, this, SLOT(deleteMe()) );
    }
}


void Smtp::deleteMe()
{
    delete this;
}

void Smtp::problemTimeOut()
{
  QString msg = "A problem has occured : the mail has not yet been sent "
    "after 20 seconds.\n";
  switch (state) {
  case Init : msg += "I'm not able to connect to your smtp server.\n"; break;
  case Mail: case Rcpt: msg += "I can connect to the machine but I can't send "
			  "my message.\n"; break; 
  case Data: case Body: msg += "The SMTP server responded but I have no answer"
			  " after sending the message.\n"; break;
  default: break;
  }
  msg += "You should send manually the file "
    "already saved in your savegame directory.";
  QMessageBox::about( qApp->activeWindow(), "QSpacehulk, Mail not send", msg);
  QTimer::singleShot( 0, this, SLOT(deleteMe()) );
}

void Smtp::dnsLookupHelper()
{
  QValueList<QHostAddress> s = dnsLookup->addresses();
  if ( s.isEmpty() && dnsLookup->isWorking() )
    return;
  
  socket->connectToHost( s.first().toString(), port.toInt() );
  t = new QTextStream( socket );
}
